@@ -104,8 +104,10 @@ namespace Banshee.Sources
         ", BansheeQuery.UriField.Column));
 
         protected HyenaSqliteCommand prune_artists_albums_command = new HyenaSqliteCommand (@"
-            DELETE FROM CoreArtists WHERE ArtistID NOT IN (SELECT ArtistID FROM CoreTracks);
-            DELETE FROM CoreAlbums WHERE AlbumID NOT IN (SELECT AlbumID FROM CoreTracks)
+            DELETE FROM CoreAlbums WHERE AlbumID NOT IN (SELECT AlbumID FROM CoreTracks);
+            DELETE FROM CoreArtists WHERE
+                    NOT EXISTS (SELECT 1 FROM CoreTracks WHERE CoreTracks.ArtistID = CoreArtists.ArtistID)
+                AND NOT EXISTS (SELECT 1 FROM CoreAlbums WHERE CoreAlbums.ArtistID = CoreArtists.ArtistID)
         ");
 
         protected HyenaSqliteCommand purge_tracks_command = new HyenaSqliteCommand (@"
@@ -244,7 +246,7 @@ namespace Banshee.Sources
         {
             PathPattern = pattern;
 
-            var file_system = PreferencesPage.Add (new Section ("file-system", Catalog.GetString ("File Organization"), 5));
+            var file_system = PreferencesPage.FindOrAdd (new Section ("file-system", Catalog.GetString ("File Organization"), 5));
             file_system.Add (new SchemaPreference<string> (pattern.FolderSchema, Catalog.GetString ("Folder hie_rarchy")));
             file_system.Add (new SchemaPreference<string> (pattern.FileSchema, Catalog.GetString ("File _name")));
         }
@@ -649,6 +651,9 @@ namespace Banshee.Sources
                     ErrorSource.AddMessage (e.Message, track.Uri.ToString ());
                 }
             }
+            if (!AddTrackJob.IsFinished) {
+                AddTrackJob.Finish ();
+            }
             is_adding = false;
         }
 
